/*	@(#)tlock.c	1.9 93/09/10 NFS Rev 2 Testsuite	*/
/*
 *	System V NFS
 *
 *	Copyright 1986, 1987, 1988, 1989 Lachman Associates, Incorporated (LAI)
 *
 *	All Rights Reserved.
 *
 *	The copyright above and this notice must be preserved in all
 *	copies of this source code.  The copyright above does not
 *	evidence any actual or intended publication of this source
 *	code.
 *
 *	This is unpublished proprietary trade secret source code of
 *	Lachman Associates.  This source code may not be copied,
 *	disclosed, distributed, demonstrated or licensed except as
 *	expressly authorized by Lachman Associates.
 */

/*
 * #ifndef lint
 * static char SysVr3NFSID[] = "@(#)tlock.c	4.11 System V NFS source";
 * #endif
 */

/*
 * Test program for System V, Release 3 record locking using lockf().
 */

#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#if defined(__STDC__)
#include <stdarg.h>
#endif
#include <sys/types.h>
#include <sys/param.h>
#include <sys/times.h>

#ifndef HZ
#define HZ	60		/* a common default */
#endif

/*
 * Implementation dependent constant denoting maximum eof.
 */
#define MAXEOF	(0x7fffffff)	/* System V.3 */

#define PARENT	0		/* Who am I? */
#define CHILD	1

#define PASS	 0		/* Passed test. */
#define EQUAL	-1		/* Compared equal. */
#define UNEQUAL	-2		/* Compared unequal. */

#define WARN	1		/* Warning, may be a problem. */
#define FATAL	2		/* Fatal testing error. */

#define END	(0)		/* A bit more readable. */

#define DO_TEST(n)	((testnum == 0) || (testnum == (n)))
#define DO_RATE(n)	((ratetest > 0) || (testnum == (n)))
#define DO_MAND(n)	((mandtest > 0) || (testnum == (n)))

int ratetest = 0;
int ratecount = 1000;
int mandtest = 0;

int wait_time = 3;

char arr[1];		/* dummy buffer for pipe */

int parentpipe[2];
int childpipe[2];

char testfile[256];	/* file for locking test */
char *filepath = ".";
int testfd;
#ifdef notdef
int open();
#endif

int testnum;	/* current test number */
int passnum;	/* current pass number */
int passcnt;	/* repeat pass count */
int cumpass;	/* pass for all passes */
int cumwarn;	/* warn for all passes */
int cumfail;	/* fail for all passes */
int tstpass;	/* pass for last pass */
int tstwarn;	/* warn for last pass */
int tstfail;	/* fail for last pass */

int parentpid, childpid;
#ifdef notdef
int fork(), getpid();
#endif
int who;

#ifdef O_SYNC
#define OPENFLAGS	(O_CREAT | O_RDWR | O_SYNC)
#else
#define OPENFLAGS	(O_CREAT | O_RDWR)
#endif
#define OPENMODES	(0666)
#define MANDMODES	(02666)

void close_testfile();
void comment();

void
initialize()
{

	parentpid = getpid();
	sprintf(&testfile[0], "%s/lockfile%d", filepath, parentpid);

	printf("Creating parent/child synchronization pipes.\n");
	pipe(parentpipe);
	pipe(childpipe);

	fflush(stdout);
}

void
testreport(nok)
	int nok;
{
	FILE *outf;
	char *sp;

	cumpass += tstpass;
	cumwarn += tstwarn;
	cumfail += tstfail;
	outf = (nok ? stderr : stdout);
	sp = ((who == PARENT) ? "PARENT" : " CHILD");
	fprintf(outf, "\n** %s pass %d results: ", sp, passnum);
	fprintf(outf, "%d/%d pass, %d/%d warn, %d/%d fail (pass/total).\n",
	       tstpass, cumpass, tstwarn, cumwarn, tstfail, cumfail);
	tstpass = tstwarn = tstfail = 0;
	fflush(outf);
}

void
testexit(nok)
	int nok;
{

	close_testfile();
	if (nok) {
		testreport(1);
	}
	if (who == PARENT) {
		signal(SIGCLD, SIG_DFL);
		if (nok) {
			signal(SIGINT, SIG_IGN);
			kill(childpid, SIGINT);
		}
		wait((int *) 0);
	} else {
		if (nok) {
			signal(SIGINT, SIG_IGN);
			kill(parentpid, SIGINT);
		} 
	}
	exit(nok);
	/* NOTREACHED */
}

/* ARGSUSED */
void
parentsig(sig)
	int sig;
{

	testexit(1);
}

/* ARGSUSED */
void
childsig(sig)
	int sig;
{

	testexit(1);
}

/* ARGSUSED */
void
childdied(sig)
	int sig;
{

	comment("Child died");
}

/* VARARGS2 */
void
header(test, fmt, arg1, arg2, arg3, arg4)
	int test;
	char *fmt;
#ifdef OSF1
	void *arg1, *arg2, *arg3, *arg4;
#else
	int arg1, arg2, arg3, arg4;
#endif
{

	printf("\nTest #%d - ", test);
	printf(fmt, arg1, arg2, arg3, arg4);
	printf("\n");
	fflush(stdout);
}

/* VARARGS1 */
void
comment(fmt, arg1, arg2, arg3, arg4)
	char *fmt;
#ifdef OSF1
	void *arg1, *arg2, *arg3, *arg4;
#else
	int arg1, arg2, arg3, arg4;
#endif
{

	printf("\t%s", ((who == PARENT) ? "Parent: " : "Child:  "));
	printf(fmt, arg1, arg2, arg3, arg4);
	printf("\n");
	fflush(stdout);
}

void
childwait()
{

	if (read(parentpipe[0], arr, 1) != 1) {
		perror("tlock: child pipe read");
		testexit(1);
	}
}

void
childfree(wait)
	int wait;
{

	if (write(parentpipe[1], arr, 1) != 1) {
		perror("tlock: childfree pipe write");
		testexit(1);
	}
	if (wait)
		sleep(wait);
}

void
parentwait()
{

	if (read(childpipe[0], arr, 1) != 1) {
		perror("tlock: parentwait pipe read");
		testexit(1);
	}
}

void
parentfree(wait)
	int wait;
{

	if (write(childpipe[1], arr, 1) != 1) {
		perror("tlock: child pipe write");
		testexit(1);
	}
	if (wait)
		sleep(wait);
}

static char tmpstr[16];

char *
terrstr(err)
	int err;
{

	switch (err) {
	case UNEQUAL:
		return("unequal");
	case EQUAL:
		return("equal");
	case PASS:
		return("success");
	case EAGAIN:
		return("EAGAIN");
	case EBADF:
		return("EBADF");
	case EACCES:
		return("EACCES");
	case EFAULT:
		return("EFAULT");
	case EINVAL:
		return("EINVAL");
	case EDEADLK:
		return("EDEADLK");
#ifdef ECOMM
	case ECOMM:
		return("ECOMM");
#endif
#ifdef ENOLINK
	case ENOLINK:
		return("ENOLINK");
#endif
	default:
		sprintf(tmpstr, "errno=%d", err);
		return(tmpstr);
	}
}

void
report(num, sec, what, offset, length, pass, result, fail)
	int num;
	int sec;
	char *what;
	off_t offset;
	long length;
	int pass;
	int result;
	int fail;
{

	printf("\t%s", ((who == PARENT) ? "Parent: " : "Child:  "));
	printf("%d.%-2d - %s [%8lx,", num, sec, what, offset);
	if (length) {
		printf("%8lx] ", length);
	} else {
		printf("  ENDING] ");
	}
	if (pass == result) {
		printf("PASSED.\n");
		tstpass++;
	} else if (fail == WARN) {
		printf("WARNING!\n");
		comment("**** Expected %s, returned %s...",
			terrstr(pass), terrstr(result));
		comment("**** Probably Sun3.2 semantics not SVVS.");
		tstwarn++;
	} else {
		printf("FAILED!\n");
		comment("**** Expected %s, returned %s...",
			terrstr(pass), terrstr(result));
		comment("**** Probably implementation error.");
		tstfail++;
		testexit(1);
	}
	fflush(stdout);
}

char *
tfunstr(fun)
	int fun;
{

	switch (fun) {
	case F_ULOCK:
		return("F_ULOCK");
	case F_LOCK:
		return("F_LOCK ");
	case F_TLOCK:
		return("F_TLOCK");
	case F_TEST:
		return("F_TEST ");
	default:
		fprintf(stderr, "tlock: unknown lockf() F_<%d>.\n", fun);
		testexit(1);
	}
	/* NOTREACHED */
}

void
open_testfile(flags, modes)
	int flags;
	int modes;
{

	testfd = open(testfile, flags, modes);
	if (testfd < 0) {
		perror("tlock: open");
		testexit(1);
	}
}

void
close_testfile()
{

	close(testfd);
	unlink(testfile);
}

void
write_testfile(datap, offset, count)
	char *datap;
	off_t offset;
	int count;
{

	(void) lseek(testfd, offset, 0);
	if (write(testfd, datap, count) != count) {
		perror("tlock: testfile write");
		testexit(1);
	}
	comment("Wrote '%s' to testfile [ %ld, %d ].",
		datap, offset, count);
}

void
read_testfile(test, sec, datap, offset, count, pass, fail)
	int test;
	int sec;
	char *datap;
	off_t offset;
	unsigned int count;
	int pass;
	int fail;
{
	int result;
	char array[64];		/* there's a limit to this... */

	(void) lseek(testfd, offset, 0);
	if (count > sizeof (array))
		count = sizeof (array);
	if ((result = read(testfd, array, count)) != count) {
		perror("tlock: testfile read");
		testexit(1);
	}
	comment("Read '%s' from testfile [ %ld, %d ].",
		datap, offset, count);
	array[count] = '\0';
	if (strncmp(datap, array, count) != 0) {
		comment("**** Test expected '%s', read '%s'.",
			datap, array);
		result = UNEQUAL;
	} else {
		result = EQUAL;
	}	
	report(test, sec, "COMPARE", offset, (long)count, pass, result, fail);
}

void
test(num, sec, func, offset, length, pass, fail)
	int num;
	int sec;
	int func;
	off_t offset;
	long length;
	int pass;
	int fail;
{
	int result;

	(void) lseek(testfd, offset, 0);
	if ((result = lockf(testfd, func, length)) != 0) {
		if (result != -1) {
			fprintf(stderr, "tlock: lockf() returned %d.\n",
				result);
			testexit(1);
		}
		result = errno;
	}
	report(num, sec, tfunstr(func), offset, length, pass, result, fail);
}

void
rate(cnt)
	int cnt;
{
	int i;
	long beg;
	long end;
#if !defined(OSF1) && !defined(HPUX) && !defined(SOLARIS2X)
	extern long times();
#endif
	struct tms tms;
	long delta;

	beg = times(&tms);
	for (i = 0; i < cnt; i++) {
		if ((lockf(testfd, F_LOCK, 1) != 0) ||
		    (lockf(testfd, F_ULOCK, 1) != 0)) {
			fprintf(stderr, "tlock: rate error=%d.\n", errno);
			tstfail++;
			break;
		}
	}
	/* See makefile: Sun must use SysV times()! */
	end = times(&tms);
	delta = ((end - beg) * 1000) / HZ;
	if (delta == 0) {
		fprintf(stderr, "tlock: rate time=0.\n");
		return;
	}
	comment("Performed %d lock/unlock cycles in %d msecs. [%d lpm].",
	       i, delta, (i * 120000) / delta);
	fflush(stdout);
}

void
test1()
{

	if (who == PARENT) {
		parentwait();
		open_testfile(OPENFLAGS, OPENMODES);
		header(1, "Test regions of an unlocked file.");
		test(1, 1, F_TEST, (off_t)0, 1L, PASS, FATAL);
		test(1, 2, F_TEST, (off_t)0, (long)END, PASS, FATAL);
		test(1, 3, F_TEST, (off_t)0, (long)MAXEOF, PASS, FATAL);
		test(1, 4, F_TEST, (off_t)1, 1L, PASS, FATAL);
		test(1, 5, F_TEST, (off_t)1, (long)END, PASS, FATAL);
		test(1, 6, F_TEST, (off_t)1, (long)MAXEOF, PASS, FATAL);
		test(1, 7, F_TEST, (off_t)MAXEOF, 1L, PASS, FATAL);
		test(1, 8, F_TEST, (off_t)MAXEOF, (long)END, PASS, FATAL);
		test(1, 9, F_TEST, (off_t)MAXEOF, (long)MAXEOF, EINVAL, WARN);
		close_testfile();
		childfree(0);
	} else {
		parentfree(0);
		childwait();
	}
}

void
test2()
{

	if (who == PARENT) {
		parentwait();
		header(2, "Try to lock the whole file.");
		open_testfile(OPENFLAGS, OPENMODES);
		test(2, 0, F_TLOCK, (off_t)0, (long)END, PASS, FATAL);
		childfree(0);
		parentwait();
		test(2, 10, F_ULOCK, (off_t)0, (long)END, PASS, FATAL);
		close_testfile();
	} else {
		parentfree(0);
		childwait();
		open_testfile(OPENFLAGS, OPENMODES);
#ifdef SVR4
		test(2, 1, F_TEST, (off_t)0, 1L, EAGAIN, WARN);
#else
		test(2, 1, F_TEST, (off_t)0, 1L, EACCES, FATAL);
#endif
		test(2, 2, F_TEST, (off_t)0, (long)END, EAGAIN, WARN);
#ifdef SVR4
		test(2, 3, F_TEST, (off_t)0, (long)MAXEOF, EAGAIN, WARN);
		test(2, 4, F_TEST, (off_t)1, 1L, EAGAIN, WARN);
		test(2, 5, F_TEST, (off_t)1, (long)END, EAGAIN, WARN);
		test(2, 6, F_TEST, (off_t)1, (long)MAXEOF, EAGAIN, WARN);
		test(2, 7, F_TEST, (off_t)MAXEOF, 1L, EAGAIN, WARN);
		test(2, 8, F_TEST, (off_t)MAXEOF, (long)END, EAGAIN, WARN);
#else
		test(2, 3, F_TEST, (off_t)0, (long)MAXEOF, EACCES, FATAL);
		test(2, 4, F_TEST, (off_t)1, 1L, EACCES, FATAL);
		test(2, 5, F_TEST, (off_t)1, (long)END, EACCES, FATAL);
		test(2, 6, F_TEST, (off_t)1, (long)MAXEOF, EACCES, FATAL);
		test(2, 7, F_TEST, (off_t)MAXEOF, 1L, EACCES, WARN);
		test(2, 8, F_TEST, (off_t)MAXEOF, (long)END, EACCES, WARN);
#endif
		test(2, 9, F_TEST, (off_t)MAXEOF, (long)MAXEOF, EINVAL, WARN);
		close_testfile();
		parentfree(0);
	}
}

void
test3()
{

	if (who == PARENT) {
		parentwait();
		header(3, "Try to lock just the 1st byte.");
		open_testfile(OPENFLAGS, OPENMODES);
		test(3, 0, F_TLOCK, (off_t)0, 1L, PASS, FATAL);
		childfree(0);
		parentwait();
		test(3, 5, F_ULOCK, (off_t)0, 1L, PASS, FATAL);
		close_testfile();
	} else {
		parentfree(0);
		childwait();
		open_testfile(OPENFLAGS, OPENMODES);
#ifdef SVR4
		test(3, 1, F_TEST, (off_t)0, 1L, EAGAIN, WARN);
		test(3, 2, F_TEST, (off_t)0, (long)END, EAGAIN, WARN);
#else
		test(3, 1, F_TEST, (off_t)0, 1L, EACCES, FATAL);
		test(3, 2, F_TEST, (off_t)0, (long)END, EACCES, FATAL);
#endif
		test(3, 3, F_TEST, (off_t)1, 1L, PASS, FATAL);
		test(3, 4, F_TEST, (off_t)1, (long)END, PASS, FATAL);
		close_testfile();
		parentfree(0);
	}
}

void
test4()
{

	if (who == PARENT) {
		parentwait();
		header(4, "Try to lock the 2nd byte, test around it.");
		open_testfile(OPENFLAGS, OPENMODES);
		test(4, 0, F_TLOCK, (off_t)1, 1L, PASS, FATAL);
		childfree(0);
		parentwait();
		test(4, 10, F_ULOCK, (off_t)1, 1L, PASS, FATAL);
		close_testfile();
	} else {
		parentfree(0);
		childwait();
		open_testfile(OPENFLAGS, OPENMODES);
		test(4, 1, F_TEST, (off_t)0, 1L, PASS, FATAL);
#ifdef SVR4
		test(4, 2, F_TEST, (off_t)0, 2L, EAGAIN, WARN);
		test(4, 3, F_TEST, (off_t)0, (long)END, EAGAIN, WARN);
		test(4, 4, F_TEST, (off_t)1, 1L, EAGAIN, WARN);
		test(4, 5, F_TEST, (off_t)1, 2L, EAGAIN, WARN);
		test(4, 6, F_TEST, (off_t)1, (long)END, EAGAIN, WARN);
#else
		test(4, 2, F_TEST, (off_t)0, 2L, EACCES, FATAL);
		test(4, 3, F_TEST, (off_t)0, (long)END, EACCES, FATAL);
		test(4, 4, F_TEST, (off_t)1, 1L, EACCES, FATAL);
		test(4, 5, F_TEST, (off_t)1, 2L, EACCES, FATAL);
		test(4, 6, F_TEST, (off_t)1, (long)END, EACCES, FATAL);
#endif
		test(4, 7, F_TEST, (off_t)2, 1L, PASS, FATAL);
		test(4, 8, F_TEST, (off_t)2, 2L, PASS, FATAL);
		test(4, 9, F_TEST, (off_t)2, (long)END, PASS, FATAL);
		close_testfile();
		parentfree(0);
	}
}

void
test5()
{

	if (who == PARENT) {
		parentwait();
		header(5, "Try to lock 1st and 2nd bytes, test around them.");
		open_testfile(OPENFLAGS, OPENMODES);
		test(5, 0, F_TLOCK, (off_t)0, 1L, PASS, FATAL);
		test(5, 1, F_TLOCK, (off_t)2, 1L, PASS, FATAL);
		childfree(0);
		parentwait();
		test(5, 14, F_ULOCK, (off_t)0, 1L, PASS, FATAL);
		test(5, 15, F_ULOCK, (off_t)2, 1L, PASS, FATAL);
		close_testfile();
	} else {
		parentfree(0);
		childwait();
		open_testfile(OPENFLAGS, OPENMODES);
#ifdef SVR4
		test(5, 2, F_TEST, (off_t)0, 1L, EAGAIN, WARN);
		test(5, 3, F_TEST, (off_t)0, 2L, EAGAIN, WARN);
		test(5, 4, F_TEST, (off_t)0, (long)END, EAGAIN, WARN);
#else
		test(5, 2, F_TEST, (off_t)0, 1L, EACCES, FATAL);
		test(5, 3, F_TEST, (off_t)0, 2L, EACCES, FATAL);
		test(5, 4, F_TEST, (off_t)0, (long)END, EACCES, FATAL);
#endif
		test(5, 5, F_TEST, (off_t)1, 1L, PASS, FATAL);
#ifdef SVR4
		test(5, 6, F_TEST, (off_t)1, 2L, EAGAIN, WARN);
		test(5, 7, F_TEST, (off_t)1, (long)END, EAGAIN, WARN);
		test(5, 8, F_TEST, (off_t)2, 1L, EAGAIN, WARN);
		test(5, 9, F_TEST, (off_t)2, 2L, EAGAIN, WARN);
		test(5, 10, F_TEST, (off_t)2, (long)END, EAGAIN, WARN);
#else
		test(5, 6, F_TEST, (off_t)1, 2L, EACCES, FATAL);
		test(5, 7, F_TEST, (off_t)1, (long)END, EACCES, FATAL);
		test(5, 8, F_TEST, (off_t)2, 1L, EACCES, FATAL);
		test(5, 9, F_TEST, (off_t)2, 2L, EACCES, FATAL);
		test(5, 10, F_TEST, (off_t)2, (long)END, EACCES, FATAL);
#endif
		test(5, 11, F_TEST, (off_t)3, 1L, PASS, FATAL);
		test(5, 12, F_TEST, (off_t)3, 2L, PASS, FATAL);
		test(5, 13, F_TEST, (off_t)3, (long)END, PASS, FATAL);
		close_testfile();
		parentfree(0);
	}
}

void
test6()
{

	if (who == PARENT) {
		parentwait();
		header(6, "Try to lock the MAXEOF byte.");
		open_testfile(OPENFLAGS, OPENMODES);
		test(6, 0, F_TLOCK, (off_t)MAXEOF, 1L, PASS, FATAL);
		childfree(0);
		parentwait();
		test(6, 10, F_ULOCK, (off_t)MAXEOF, 1L, PASS, FATAL);
		close_testfile();
	} else {
		parentfree(0);
		childwait();
		open_testfile(OPENFLAGS, OPENMODES);
		test(6, 1, F_TEST, (off_t)MAXEOF - 1, 1L, PASS, FATAL);
#ifdef SVR4
		test(6, 2, F_TEST, (off_t)MAXEOF - 1, 2L, EAGAIN, WARN);
		test(6, 3, F_TEST, (off_t)MAXEOF - 1, (long)END, EAGAIN, WARN);
		test(6, 4, F_TEST, (off_t)MAXEOF, 1L, EAGAIN, WARN);
#else
		test(6, 2, F_TEST, (off_t)MAXEOF - 1, 2L, EACCES, FATAL);
		test(6, 3, F_TEST, (off_t)MAXEOF - 1, (long)END, EACCES, WARN);
		test(6, 4, F_TEST, (off_t)MAXEOF, 1L, EACCES, FATAL);
#endif
		test(6, 5, F_TEST, (off_t)MAXEOF, 2L, EINVAL, WARN);
#ifdef SVR4
		test(6, 6, F_TEST, (off_t)MAXEOF, (long)END, EAGAIN, WARN);
		test(6, 7, F_TEST, (off_t)((unsigned long)MAXEOF + 1), 1L, EAGAIN, WARN);
#else
		test(6, 6, F_TEST, (off_t)MAXEOF, (long)END, EACCES, FATAL);
		test(6, 7, F_TEST, (off_t)((unsigned long)MAXEOF + 1), 1L, EACCES, WARN);
#endif
		test(6, 8, F_TEST, (off_t)((unsigned long)MAXEOF + 1), 2L, EINVAL, FATAL);
#ifdef SVR4
		test(6, 9, F_TEST, (off_t)((unsigned long)MAXEOF + 1), (long)END, EAGAIN, WARN);
#else
		test(6, 9, F_TEST, (off_t)((unsigned long)MAXEOF + 1), (long)END, EACCES, WARN);
#endif
		close_testfile();
		parentfree(0);
	}
}

void
test7()
{

	if (who == PARENT) {
		parentwait();
		header(7, "Test parent/child mutual exclusion.");
		open_testfile(OPENFLAGS, OPENMODES);
		test(7, 0, F_TLOCK, (off_t)0, 4L, PASS, FATAL);
		write_testfile("aaaa", (off_t)0, 4);
		comment("Now free child to run, should block on lock.");
		childfree(wait_time);
		comment("Check data in file to insure child blocked.");
		read_testfile(7, 1, "aaaa", (off_t)0, 4, EQUAL, FATAL);
		comment("Now unlock region so child will unblock.");
		test(7, 2, F_ULOCK, (off_t)0, 4L, PASS, FATAL);
		comment("Now try to regain lock, parent should block.");
		test(7, 5, F_LOCK, (off_t)0, 4L, PASS, FATAL);
		comment("Check data in file to insure child unblocked.");
		read_testfile(7, 6, "bbbb", (off_t)0, 4, EQUAL, FATAL);
		test(7, 7, F_ULOCK, (off_t)0, 4L, PASS, FATAL);
		close_testfile();
		childfree(0);
	} else {
		parentfree(0);
		childwait();
		open_testfile(OPENFLAGS, OPENMODES);
		test(7, 3, F_LOCK, (off_t)0, 4L, PASS, FATAL);
		write_testfile("bbbb", (off_t)0, 4);
		test(7, 4, F_ULOCK, (off_t)0, 4L, PASS, FATAL);
		close_testfile();
		childwait();
	}
}

void
test8()
{

	if (who == PARENT) {
		parentwait();
		header(8, "Rate test performing lock/unlock cycles.");
		open_testfile(OPENFLAGS, OPENMODES);
		rate(ratecount);
		close_testfile();
		childfree(0);
	} else {
		parentfree(0);
		childwait();
	}
}

void
test9()
{

	if (who == PARENT) {
		parentwait();
		header(9, "Test mandatory locking (LAI NFS client).");
		open_testfile(OPENFLAGS, MANDMODES);
		test(9, 0, F_TLOCK, (off_t)0, 4L, PASS, FATAL);
		write_testfile("aaaa", (off_t)0, 4);
		comment("Now free child to run, should block on write.");
		childfree(wait_time);
		comment("Check data in file before child writes.");
		read_testfile(9, 1, "aaaa", (off_t)0, 4, EQUAL, FATAL);
		comment("Now unlock region so child will write.");
		test(9, 2, F_ULOCK, (off_t)0, 4L, PASS, FATAL);
		comment("Check data in file after child writes.");
		read_testfile(9, 3, "bbbb", (off_t)0, 4, EQUAL, FATAL);
		close_testfile();
		childfree(0);
	} else {
		parentfree(0);
		childwait();
		open_testfile(OPENFLAGS, OPENMODES);
		write_testfile("bbbb", (off_t)0, 4);
		close_testfile();
		childwait();
	}
}

void
test10()
{

	if (who == PARENT) {
		parentwait();
		header(10, "Make sure a locked region is split properly.");
		open_testfile(OPENFLAGS, OPENMODES);
		test(10, 0, F_TLOCK, (off_t)0, 3L, PASS, FATAL);
		test(10, 1, F_ULOCK, (off_t)1, 1L, PASS, FATAL);
		childfree(0);
		parentwait();
		test(10, 6, F_ULOCK, (off_t)0, 1L, PASS, FATAL);
		test(10, 7, F_ULOCK, (off_t)2, 1L, PASS, FATAL);
		childfree(0);
		parentwait();
		test(10, 9, F_ULOCK, (off_t)0, 1L, PASS, WARN);
		test(10, 10, F_TLOCK, (off_t)1, 3L, PASS, WARN);
		test(10, 11, F_ULOCK, (off_t)2, 1L, PASS, WARN);
		childfree(0);
		parentwait();
		close_testfile();
	} else {
		parentfree(0);
		childwait();
		open_testfile(OPENFLAGS, OPENMODES);
#ifdef SVR4
		test(10, 2, F_TEST, (off_t)0, 1L, EAGAIN, WARN);
		test(10, 3, F_TEST, (off_t)2, 1L, EAGAIN, WARN);
#else
		test(10, 2, F_TEST, (off_t)0, 1L, EACCES, WARN);
		test(10, 3, F_TEST, (off_t)2, 1L, EACCES, WARN);
#endif
		test(10, 4, F_TEST, (off_t)3, (long)END, PASS, WARN);
		test(10, 5, F_TEST, (off_t)1, 1L, PASS, WARN);
		parentfree(0);
		childwait();
		test(10, 8, F_TEST, (off_t)0, 3L, PASS, WARN);
		parentfree(0);
		childwait();
#ifdef SVR4
		test(10, 12, F_TEST, (off_t)1, 1L, EAGAIN, WARN);
		test(10, 13, F_TEST, (off_t)3, 1L, EAGAIN, WARN);
#else
		test(10, 12, F_TEST, (off_t)1, 1L, EACCES, WARN);
		test(10, 13, F_TEST, (off_t)3, 1L, EACCES, WARN);
#endif
		test(10, 14, F_TEST, (off_t)4, (long)END, PASS, WARN);
		test(10, 15, F_TEST, (off_t)2, 1L, PASS, WARN);
		test(10, 16, F_TEST, (off_t)0, 1L, PASS, WARN);
		close_testfile();
		parentfree(0);
	}
}

void
runtests()
{

	if (DO_TEST(1)) {
		test1();
	}
	if (DO_TEST(2)) {
		test2();
	}
	if (DO_TEST(3)) {
		test3();
	}
	if (DO_TEST(4)) {
		test4();
	}
	if (DO_TEST(5)) {
		test5();
	}
	if (DO_TEST(6)) {
		test6();
	}
	if (DO_TEST(7)) {
		test7();
	}
	if (DO_RATE(8)) {
		test8();
	}
	if (DO_MAND(9)) {
		test9();
	}
	if (DO_TEST(10)) {
		test10();
	}
}

/*
 * Main record locking test loop.
 */
main(argc, argv)
	int argc;
	char **argv;
{
	int c;
	extern int optind;
	extern char *optarg;
	int errflg = 0;

	passcnt = 1;	/* default, test for 1 pass */

	while ((c = getopt(argc, argv, "p:t:rmw:")) != -1) {
		switch (c) {
		case 'p':
			sscanf(optarg, "%d", &passcnt);
			break;
		case 't':
			sscanf(optarg, "%d", &testnum);
			break;
		case 'r':
			ratetest++;
			break;
		case 'm':
			mandtest++;
			break;
		case 'w':
			sscanf(optarg, "%d", &wait_time);
			break;
		default:
			errflg++;
		}
	}
	if (errflg) {
		fprintf(stderr, "usage: tlock [-p passcnt] [-t testnum] [-r] [-m] [-w wait_time] [dirpath]\n");
		exit(2);
	}
	if (optind < argc) {
		filepath = argv[optind];
	}
	initialize();

	/*
	 * Fork child...
	 */
	if ((childpid = fork()) == 0) {
		who = CHILD;
		signal(SIGINT, parentsig);
	} else {
		who = PARENT;
		signal(SIGINT, childsig);
		signal(SIGCLD, childdied);
	}

	/*
	 * ...and run the tests for count passes.
	 */
	for (passnum = 1; passnum <= passcnt; passnum++) {
		runtests();
		if (who == CHILD) {
			childwait();
			testreport(0);
		} else {
			testreport(0);
			childfree(0);
		}
	}
	if (who == CHILD) {
		childwait();
	} else {
		signal(SIGCLD, SIG_DFL);
		childfree(0);
	}
	testexit(0);
	/* NOTREACHED */
}
