/*
 * Rename a file
 *
 * -T delay	Initial delay before starting to rename
 * -t delay	Sleep for delay seconds after each rename
 *
 * Copyright (C) 1998, Olaf Kirch <okir@monad.swb.de>
 */
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <ctype.h>
#include <sys/wait.h>

static void
usage(int code)
{
	printf("usage:\n"
		"rename [-t delay] file1 file2 ...\n");
	exit(code);
}

int
main(int argc, char **argv)
{
	char	**files;
	int	initdelay = 0, delay = 0, nrfiles, nr, next, c;

	while ((c = getopt(argc, argv, "T:t:h")) != -1) {
		switch (c) {
		case 'T':
			initdelay = atoi(optarg);
			break;
		case 't':
			delay = atoi(optarg);
			break;
		case 'h':
			usage(0);
		case '?':
			usage(1);
		}
	}

	if (optind >= argc - 2) {
		printf("Missing file names\n");
		usage(1);
	}

	files = argv + optind;
	nrfiles = argc - optind;

	if (initdelay)
		sleep(initdelay);

	nr = 0;
	while (1) {
		next = (nr + 1) % nrfiles;
		if (rename(files[nr], files[next]) < 0) {
			perror("rename");
			exit(1);
		}
		if (delay)
			sleep(delay);
		nr = next;
	}

	return 0;
}
